<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Remessa;

use Doctrine\ORM\EntityManager;
use \Financeiro\Entities\Cnabrec;
use Financeiro\Entities\Emprbanc;
use Geral\Entities\Contcorr;
use Teknisa\FIN\Modules\ContaReceber\Factory\TitulosAReceberFactory;
use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco\GeraCnabSantanderService;
use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco\GeraCnabBradescoService;
use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco\GeraCnabItauService;
use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco\GeraCnabBrasilService;
use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco\GeraCnabCaixaService;
use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco\GeraCnabSicoobService;
use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco\GeraCnabSicrediService;
use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco\GeraCnabSofisaService;
use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco\GeraCnabVotorantimService;

/**
 * Class RemessaReceberService
 * @package Teknisa\FIN\Modules\ContaReceber\Services
 */
abstract class GeraCnabService{

	const CODIGO_COMPENSACAO_SANTANDER   = "033";
    const CODIGO_COMPESACAO_BRADESCO     = "237";
    const CODIGO_COMPESACAO_ITAU         = "341";
    const CODIGO_COMPESACAO_BRASIL       = "001";
    const CODIGO_COMPESACAO_CAIXA        = "104";
    const CODIGO_COMPESACAO_SICOOB       = "756";
    const CODIGO_COMPESACAO_SICREDI      = "748";
    const CODIGO_COMPESACAO_SOFISA       = "637";
    const CODIGO_COMPESACAO_VOTORANTIM   = "655";

    /**
     * @param Cnabrec $remessa
     * @return null
     */
	public static function factory(
        Cnabrec $remessa, EntityManager $entitymanager, TitulosAReceberFactory $titulosareceberfactory
    ){
        $codigoCompensacao = $remessa->getReducont()->getAgencia()->getBanco()->getCdbanccnab();
        if ($codigoCompensacao === self::CODIGO_COMPENSACAO_SANTANDER) {
            return new GeraCnabSantanderService($entitymanager, $titulosareceberfactory);
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPESACAO_BRADESCO) {
            return new GeraCnabBradescoService($remessa);
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPESACAO_ITAU) {
            return new GeraCnabItauService($remessa);
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPESACAO_BRASIL) {
            return new GeraCnabBrasilService($remessa);
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPESACAO_CAIXA) {
            return new GeraCnabCaixaService($remessa);
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPESACAO_SICOOB) {
            return new GeraCnabSicoobService($remessa);
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPESACAO_SICREDI) {
            return new GeraCnabSicrediService($remessa);
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPESACAO_SOFISA) {
            return new GeraCnabSofisaService($remessa);
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPESACAO_VOTORANTIM) {
            return new GeraCnabVotorantimService($remessa);
        }

        return null;
    }

    /**
     * @param $cdreducont
     * @param $cdbancocnab
     * @param $cdempresa
     * @return string
     */
    protected function getIdContRec(
        $cdreducont, $cdbancocnab, $cdempresa
    ){
        /** @var $contcorr Contcorr */
        $contcorr = Contcorr::findOneByCdreducont($cdreducont);
        
        if($contcorr and $contcorr->getCdidreccontcorr()){
            $resultado =  $contcorr->getCdidreccontcorr();
        }else{
            $emprbanc = Emprbanc::findOneByCdempresaAndCdbanco($cdempresa, $cdbancocnab);

            if($emprbanc and $emprbanc->getCdempbancrec())
                 $resultado = $emprbanc->getCdempbancrec();
            else $resultado = '';
        }

        return $resultado;
    }

    /**
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $nrseqremessarec
     * @param $nrcnab
     * @param $cdreducont
     * @param null $carteira
     * @return mixed
     */
	public abstract function geraHeaderArquivo(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec,
        $nrseqremessarec, $nrcnab, $cdreducont, $carteira = NULL, $contreg = NULL, $idtipcnabrec = NULL, $cdfilial, $contador = NULL
    );

    /**
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $nrseqremessarec
     * @param $nrcnabrec
     * @param $cdreducont
     * @param null $carteira
     * @return mixed
     */
	public abstract function geraHeaderLote(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec,
        $nrseqremessarec, $nrcnabrec, $cdreducont, $carteira = NULL, $cdfilial
    );

    /**
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $nrseqremessarec
     * @param $nrcnabrec
     * @param $idoperacao
     * @param $cdagenciacnab
     * @param $cdcontacorrente
     * @param $cdcontcorrcobranca
     * @param $nossonumero
     * @param $idtipocobranca
     * @param $nrlanbolrec
     * @param $cdfilial
     * @param $dtvenbolrec
     * @param $valorfinalboleto
     * @param $valorpemobrec
     * @param $valormordiabrec
     * @param $valorpemulbrec
     * @param $valormultabrec
     * @param $valorfinalbrec
     * @param $cdcartcobbol
     * @param $cddiaprotbol
     * @param $cdreducont
     * @param $imprbol
     * @param $qtdiaprotbol
     * @return mixed
     */
	public abstract function geraDetalheSegmentoP(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnabrec,
        $idoperacao, $cdagenciacnab, $cdcontacorrente, $cdcontcorrcobranca, $nossonumero, $idtipocobranca,
        $nrlanbolrec, $cdfilial, $dtvenbolrec, $valorfinalboleto, $valorpemobrec, $valormordiabrec, $valorpemulbrec,
        $valormultabrec, $valorfinalbrec, $cdcartcobbol, $cddiaprotbol, $cdreducont, $imprbol, $qtdiaprotbol, $contador = NULL
    );

    /**
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $cdcliente
     * @param $idoperacao
     * @param $cdempresa
     * @param $cdfilial
     * @param $nrlanbolrec
     * @return mixed
     */
	public abstract function geraDetalheSegmentoQ(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $cdcliente, $idoperacao,
        $cdempresa, $cdfilial, $nrlanbolrec, $contador = NULL
    );

    /**
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param $idoperacao
     * @param $dtvenbolrec
     * @param $valorpemulbrec
     * @param $nrlanbolrec
     * @param $valormultabrec
     * @return mixed
     */
    public abstract function geraDetalheSegmentoR(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $dtvenbolrec,
        $valorpemulbrec, $nrlanbolrec, $valormultabrec, $contador = NULL
    );

    /**
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param null $contreg
     * @return mixed
     */
    public abstract function geraTraillerLote(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $contreg = NULL
    );

    /**
     * @param $caminho
     * @param $nmarquivo
     * @param $nrorg
     * @param $cdbanco
     * @param null $contlote
     * @param null $contreg
     * @return mixed
     */
    public abstract function geraTrailler(
        $caminho, $nmarquivo, $nrorg, $cdbanco, $contlote = NULL, $contreg = NULL, $cdreducont = NULL
    );

    /**
     * @param $caminho
     * @param $nmarquivo
     * @param $cdempresa
     * @param $cdcartcobbol
     * @param $cdagencia
     * @param $cdcontcorr
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $vrpemobrec
     * @param $idimprboleto
     * @param $cdnosnumrec
     * @param $dtvenbolrec
     * @param $cdins1cobbol
     * @param $qtdiaprotbol
     * @param $vrmordiabrec
     * @param $vrfinalbrec
     * @param $idopecnabrec
     * @param $vrpemulbrec
     * @param $idtpijurclie
     * @param $nrinsjurclie
     * @param $cdcliente
     * @param $cdins2cobbol
     * @return mixed
     */
    public abstract function geraTransacao(
        $caminho, $nmarquivo, $cdempresa,$nrorg, $cdcartcobbol, $cdagencia, $cdcontcorr, $cdfilial,
        $nrlanbolrec, $vrpemobrec, $idimprboleto, $cdnosnumrec, $dtvenbolrec, $cdins1cobbol,
        $qtdiaprotbol, $vrmordiabrec, $vrfinalbrec, $idopecnabrec, $vrpemulbrec, $idtpijurclie,
        $nrinsjurclie, $cdcliente, $cdins2cobbol, $contreg = NULL, $idtipcnabrec = null, $idfilialcnabrec, $cddiaprotbol = NULL, $contador = NULL,$cdreducont,$cdbanco
    );
}											 